from rs_datasets.movielens import MovieLens
from rs_datasets.msd import MillionSongDataset
from rs_datasets.netflix import Netflix
from rs_datasets.lastfm import Lastfm
from rs_datasets.epinions import Epinions
from rs_datasets.goodreads import Goodreads
from rs_datasets.dating_agency import DatingAgency
from rs_datasets.book_crossing import BookCrossing
from rs_datasets.jester import Jester
from rs_datasets.amazon import Amazon
